with Mapcode_Utils.As_U;
use Mapcode_Utils;
package Countries is

  -- The public part of the definition of a territory
  type Territory_Def is record
    -- The number, most efficient way to denote the territory
    Num  : String (1 .. 3);
    -- The ISO 3166 Alpha code
    Code : As_U.Asu_Us;
    -- The full name
    Name : As_U.Asu_Us;
  end record;

  -- All the territories supported by Madpcodes
  Territories_Def : constant array (Positive range <>) of Territory_Def := (
    ("000", As_U.Tus ("VAT"),
            As_U.Tus ("Vatican (Holy See) (_ City State)")),
    ("001", As_U.Tus ("MCO"),
            As_U.Tus ("Monaco (Principality of _)")),
    ("002", As_U.Tus ("GIB"),
            As_U.Tus ("Gibraltar")),
    ("003", As_U.Tus ("TKL"),
            As_U.Tus ("Tokelau")),
    ("004", As_U.Tus ("CCK"),
            As_U.Tus ("Cocos Islands (Keeling Islands)")),
    ("005", As_U.Tus ("BLM"),
            As_U.Tus ("Saint-Barthelemy (Collectivity of _)")),
    ("006", As_U.Tus ("NRU"),
            As_U.Tus ("Nauru (Republic of _)")),
    ("007", As_U.Tus ("TUV"),
            As_U.Tus ("Tuvalu")),
    ("008", As_U.Tus ("MAC"),
            As_U.Tus ("Macau (Aomen)")),
    ("009", As_U.Tus ("SXM"),
            As_U.Tus ("Sint Maarten")),
    ("010", As_U.Tus ("MAF"),
            As_U.Tus ("Saint Martin (Collectivity of _)")),
    ("011", As_U.Tus ("NFK"),
            As_U.Tus ("Norfolk and Philip Island"
                    & " (Philip Island) (Norfolk Island)")),
    ("012", As_U.Tus ("PCN"),
            As_U.Tus ("Pitcairn Islands"
                    & " (Pitcairn, Henderson, Ducie and Oeno Islands)")),
    ("013", As_U.Tus ("BVT"),
            As_U.Tus ("Bouvet Island")),
    ("014", As_U.Tus ("BMU"),
            As_U.Tus ("Bermuda (Somers Isles)")),
    ("015", As_U.Tus ("IOT"),
            As_U.Tus ("British Indian Ocean Territory")),
    ("016", As_U.Tus ("SMR"),
            As_U.Tus ("San Marino (Republic of _)")),
    ("017", As_U.Tus ("GGY"),
            As_U.Tus ("Guernsey (Bailiwick of _)")),
    ("018", As_U.Tus ("AIA"),
            As_U.Tus ("Anguilla")),
    ("019", As_U.Tus ("MSR"),
            As_U.Tus ("Montserrat")),
    ("020", As_U.Tus ("JEY"),
            As_U.Tus ("Jersey (Bailiwick of _)")),
    ("021", As_U.Tus ("CXR"),
            As_U.Tus ("Christmas Island")),
    ("022", As_U.Tus ("WLF"),
            As_U.Tus ("Wallis and Futuna"
                    & " (Futuna) (Wallis) (Collectivity of the _ Islands)")),
    ("023", As_U.Tus ("VGB"),
            As_U.Tus ("British Virgin Islands (Virgin Islands, British)")),
    ("024", As_U.Tus ("LIE"),
            As_U.Tus ("Liechtenstein (Principality of _)")),
    ("025", As_U.Tus ("ABW"),
            As_U.Tus ("Aruba")),
    ("026", As_U.Tus ("MHL"),
            As_U.Tus ("Marshall Islands (Republic of the _)")),
    ("027", As_U.Tus ("ASM"),
            As_U.Tus ("American Samoa (Samoa, American)")),
    ("028", As_U.Tus ("COK"),
            As_U.Tus ("Cook Islands")),
    ("029", As_U.Tus ("SPM"),
            As_U.Tus ("Saint Pierre and Miquelon"
                    & " (Miquelon) (Saint Pierre) (Collectivity of _)")),
    ("030", As_U.Tus ("NIU"),
            As_U.Tus ("Niue")),
    ("031", As_U.Tus ("KNA"),
            As_U.Tus ("Saint Kitts and Nevis"
                    & " (Nevis) (Saint Kitts) (Federation of _)")),
    ("032", As_U.Tus ("CYM"),
            As_U.Tus ("Cayman islands")),
    ("033", As_U.Tus ("BES"),
            As_U.Tus ("Bonaire, St Eustasuis and Saba"
                    & " (Bonaire) (Saba) (St Eustasius)")),
    ("034", As_U.Tus ("MDV"),
            As_U.Tus ("Maldives (Republic of _)")),
    ("035", As_U.Tus ("SHN"),
            As_U.Tus ("Saint Helena, Ascension and Tristan da Cunha"
                    & " (Saint Helena) (Ascension) (Tristan da Cunha)")),
    ("036", As_U.Tus ("MLT"),
            As_U.Tus ("Malta (Republic of _)")),
    ("037", As_U.Tus ("GRD"),
            As_U.Tus ("Grenada")),
    ("038", As_U.Tus ("VIR"),
            As_U.Tus ("Virgin Islands of the United States"
                    & " (US Virgin Islands) (American Virgin Islands)")),
    ("039", As_U.Tus ("MYT"),
            As_U.Tus ("Mayotte (Maore)")),
    ("040", As_U.Tus ("SJM"),
            As_U.Tus ("Svalbard and Jan Mayen"
                    & " (Svalbard) (Jan Mayen) (Spitsbergen)")),
    ("041", As_U.Tus ("VCT"),
            As_U.Tus ("Saint Vincent and the Grenadines"
                    & " (Saint Vincent) (Grenadines)")),
    ("042", As_U.Tus ("HMD"),
            As_U.Tus ("Heard Island and McDonald Islands"
                    & " (Heard Island) (McDonald Islands)")),
    ("043", As_U.Tus ("BRB"),
            As_U.Tus ("Barbados")),
    ("044", As_U.Tus ("ATG"),
            As_U.Tus ("Antigua and Barbuda (Antigua) (Barbuda)")),
    ("045", As_U.Tus ("CUW"),
            As_U.Tus ("Curacao")),
    ("046", As_U.Tus ("SYC"),
            As_U.Tus ("Seychelles (Republic of _)")),
    ("047", As_U.Tus ("PLW"),
            As_U.Tus ("Palau (Republic of _)")),
    ("048", As_U.Tus ("MNP"),
            As_U.Tus ("Northern Mariana Islands (Commonwealth of the _)")),
    ("049", As_U.Tus ("AND"),
            As_U.Tus ("Andorra"
                    & " (Principality of _)"
                    & " (Principality of the Valleys of _)")),
    ("050", As_U.Tus ("GUM"),
            As_U.Tus ("Guam")),
    ("051", As_U.Tus ("IMN"),
            As_U.Tus ("Isle of Mann (Mann)")),
    ("052", As_U.Tus ("LCA"),
            As_U.Tus ("Saint Lucia")),
    ("053", As_U.Tus ("FSM"),
            As_U.Tus ("Micronesia (Federated States of Micronesia)")),
    ("054", As_U.Tus ("SGP"),
            As_U.Tus ("Singapore (Republic of _)")),
    ("055", As_U.Tus ("TON"),
            As_U.Tus ("Tonga (Kingdom of _)")),
    ("056", As_U.Tus ("DMA"),
            As_U.Tus ("Dominica (Commonwealth of _)")),
    ("057", As_U.Tus ("BHR"),
            As_U.Tus ("Bahrain (Kingdom of _)")),
    ("058", As_U.Tus ("KIR"),
            As_U.Tus ("Kiribati (Republic of _)")),
    ("059", As_U.Tus ("TCA"),
            As_U.Tus ("Turks and Caicos Islands"
                    & " (Turks Islands) (Caicos Islands)")),
    ("060", As_U.Tus ("STP"),
            As_U.Tus ("Sao Tome and Principe"
                    & " (Sao Tome) (Principe) (Democratic Republic of _)")),
    ("061", As_U.Tus ("HKG"),
            As_U.Tus ("Hong Kong (Xianggang)")),
    ("062", As_U.Tus ("MTQ"),
            As_U.Tus ("Martinique")),
    ("063", As_U.Tus ("FRO"),
            As_U.Tus ("Faroe Islands")),
    ("064", As_U.Tus ("GLP"),
            As_U.Tus ("Guadeloupe")),
    ("065", As_U.Tus ("COM"),
            As_U.Tus ("Comoros (Union of the _)")),
    ("066", As_U.Tus ("MUS"),
            As_U.Tus ("Mauritius (Republic of _)")),
    ("067", As_U.Tus ("REU"),
            As_U.Tus ("Reunion")),
    ("068", As_U.Tus ("LUX"),
            As_U.Tus ("Luxembourg (Grand Duchy of _)")),
    ("069", As_U.Tus ("WSM"),
            As_U.Tus ("Samoa (Independent State of _)")),
    ("070", As_U.Tus ("SGS"),
            As_U.Tus ("South Georgia and the South Sandwich Islands"
                    & " (South Georgia) (South Sandwich Islands)")),
    ("071", As_U.Tus ("PYF"),
            As_U.Tus ("French Polynesia (Collectivity of _)")),
    ("072", As_U.Tus ("CPV"),
            As_U.Tus ("Cape Verde (Cabo Verde) (Republic of Cabo Verde)")),
    ("073", As_U.Tus ("TTO"),
            As_U.Tus ("Trinidad and Tobago"
                    & " (Republic of _) (Trinidad) (Tobago)")),
    ("074", As_U.Tus ("BRN"),
            As_U.Tus ("Brunei (Nation of _, the Abode of Peace)")),
    ("075", As_U.Tus ("ATF"),
            As_U.Tus ("French Southern and Antarctic Lands")),
    ("076", As_U.Tus ("PRI"),
            As_U.Tus ("Puerto Rico (Commonwealth of _)")),
    ("077", As_U.Tus ("CYP"),
            As_U.Tus ("Cyprus (Republic of _)")),
    ("078", As_U.Tus ("LBN"),
            As_U.Tus ("Lebanon (Lebanese Republic)")),
    ("079", As_U.Tus ("JAM"),
            As_U.Tus ("Jamaica")),
    ("080", As_U.Tus ("GMB"),
            As_U.Tus ("Gambia (The Gambia) (Republic of the _)")),
    ("081", As_U.Tus ("QAT"),
            As_U.Tus ("Qatar (State of _)")),
    ("082", As_U.Tus ("FLK"),
            As_U.Tus ("Falkland Islands (The Falklands)")),
    ("083", As_U.Tus ("VUT"),
            As_U.Tus ("Vanuatu (Republic of _)")),
    ("084", As_U.Tus ("MNE"),
            As_U.Tus ("Montenegro")),
    ("085", As_U.Tus ("BHS"),
            As_U.Tus ("Bahamas (Commonwealth of the _)")),
    ("086", As_U.Tus ("TLS"),
            As_U.Tus ("Timor-Leste (Democratic Republic of _) (East Timor)")),
    ("087", As_U.Tus ("SWZ"),
            As_U.Tus ("Swaziland (Kingdom of _)")),
    ("088", As_U.Tus ("KWT"),
            As_U.Tus ("Kuwait (State of _)")),
    ("089", As_U.Tus ("FJI"),
            As_U.Tus ("Fiji (Republic of _)")),
    ("090", As_U.Tus ("NCL"),
            As_U.Tus ("New Caledonia")),
    ("091", As_U.Tus ("SVN"),
            As_U.Tus ("Slovenia (Republic of _)")),
    ("092", As_U.Tus ("ISR"),
            As_U.Tus ("Israel (State of _)")),
    ("093", As_U.Tus ("PSE"),
            As_U.Tus ("Palestinian territories (State of Palestine)")),
    ("094", As_U.Tus ("SLV"),
            As_U.Tus ("El Salvador (Republic of _)")),
    ("095", As_U.Tus ("BLZ"),
            As_U.Tus ("Belize")),
    ("096", As_U.Tus ("DJI"),
            As_U.Tus ("Djibouti (Republic of _)")),
    ("097", As_U.Tus ("MKD"),
            As_U.Tus ("Macedonia"
                    & " (Republic of _) (FYROM)"
                    & " (Former Yugoslav Republic of Macedonia)")),
    ("098", As_U.Tus ("RWA"),
            As_U.Tus ("Rwanda (Republic of _)")),
    ("099", As_U.Tus ("HTI"),
            As_U.Tus ("Haiti (Republic of _)")),
    ("100", As_U.Tus ("BDI"),
            As_U.Tus ("Burundi (Republic of _)")),
    ("101", As_U.Tus ("GNQ"),
            As_U.Tus ("Equatorial Guinea (Republic of _)")),
    ("102", As_U.Tus ("ALB"),
            As_U.Tus ("Albania (Republic of _)")),
    ("103", As_U.Tus ("SLB"),
            As_U.Tus ("Solomon Islands")),
    ("104", As_U.Tus ("ARM"),
            As_U.Tus ("Armenia (Republic of _)")),
    ("105", As_U.Tus ("LSO"),
            As_U.Tus ("Lesotho (Kingdom of _)")),
    ("106", As_U.Tus ("BEL"),
            As_U.Tus ("Belgium (Kingdom of _)")),
    ("107", As_U.Tus ("MDA"),
            As_U.Tus ("Moldova (Republic of _)")),
    ("108", As_U.Tus ("GNB"),
            As_U.Tus ("Guinea-Bissau (Republic of _)")),
    ("109", As_U.Tus ("TWN"),
            As_U.Tus ("Taiwan (Republic of China)")),
    ("110", As_U.Tus ("BTN"),
            As_U.Tus ("Bhutan (Kingdom of _)")),
    ("111", As_U.Tus ("CHE"),
            As_U.Tus ("Switzerland (Swiss Confederation)")),
    ("112", As_U.Tus ("NLD"),
            As_U.Tus ("Netherlands (The Netherlands) (Kingdom of the _)")),
    ("113", As_U.Tus ("DNK"),
            As_U.Tus ("Denmark (Kingdom of _)")),
    ("114", As_U.Tus ("EST"),
            As_U.Tus ("Estonia (Republic of _)")),
    ("115", As_U.Tus ("DOM"),
            As_U.Tus ("Dominican Republic")),
    ("116", As_U.Tus ("SVK"),
            As_U.Tus ("Slovakia (Slovak Republic)")),
    ("117", As_U.Tus ("CRI"),
            As_U.Tus ("Costa Rica (Republic of _)")),
    ("118", As_U.Tus ("BIH"),
            As_U.Tus ("Bosnia and Herzegovina")),
    ("119", As_U.Tus ("HRV"),
            As_U.Tus ("Croatia (Republic of _)")),
    ("120", As_U.Tus ("TGO"),
            As_U.Tus ("Togo (Togolese Republic)")),
    ("121", As_U.Tus ("LVA"),
            As_U.Tus ("Latvia (Republic of _)")),
    ("122", As_U.Tus ("LTU"),
            As_U.Tus ("Lithuania (Republic of _)")),
    ("123", As_U.Tus ("LKA"),
            As_U.Tus ("Sri Lanka (Democratic Socialist Republic of _)")),
    ("124", As_U.Tus ("GEO"),
            As_U.Tus ("Georgia")),
    ("125", As_U.Tus ("IRL"),
            As_U.Tus ("Ireland (Republic of _)")),
    ("126", As_U.Tus ("SLE"),
            As_U.Tus ("Sierra Leone (Republic of _)")),
    ("127", As_U.Tus ("PAN"),
            As_U.Tus ("Panama (Republic of _)")),
    ("128", As_U.Tus ("CZE"),
            As_U.Tus ("Czech Republic")),
    ("129", As_U.Tus ("GUF"),
            As_U.Tus ("French Guiana (Guiana)")),
    ("130", As_U.Tus ("ARE"),
            As_U.Tus ("United Arab Emirates (Emirates)")),
    ("131", As_U.Tus ("AUT"),
            As_U.Tus ("Austria (Republic of _)")),
    ("132", As_U.Tus ("AZE"),
            As_U.Tus ("Azerbaijan (Republic of _)")),
    ("133", As_U.Tus ("SRB"),
            As_U.Tus ("Serbia (Republic of _)")),
    ("134", As_U.Tus ("JOR"),
            As_U.Tus ("Jordan (Hashemite Kingdom of _) (Kingdom of _)")),
    ("135", As_U.Tus ("PRT"),
            As_U.Tus ("Portugal (Portuguese Republic)")),
    ("136", As_U.Tus ("HUN"),
            As_U.Tus ("Hungary (Republic of _)")),
    ("137", As_U.Tus ("KOR"),
            As_U.Tus ("South Korea (Republic of Korea) (Korea, South)")),
    ("138", As_U.Tus ("ISL"),
            As_U.Tus ("Iceland")),
    ("139", As_U.Tus ("GTM"),
            As_U.Tus ("Guatemala (Republic of _)")),
    ("140", As_U.Tus ("CUB"),
            As_U.Tus ("Cuba (Republic of _)")),
    ("141", As_U.Tus ("BGR"),
            As_U.Tus ("Bulgaria (Republic of _)")),
    ("142", As_U.Tus ("LBR"),
            As_U.Tus ("Liberia (Republic of _)")),
    ("143", As_U.Tus ("HND"),
            As_U.Tus ("Honduras (Republic of _)")),
    ("144", As_U.Tus ("BEN"),
            As_U.Tus ("Benin (Republic of _)")),
    ("145", As_U.Tus ("ERI"),
            As_U.Tus ("Eritrea (State of _)")),
    ("146", As_U.Tus ("MWI"),
            As_U.Tus ("Malawi (Republic of _)")),
    ("147", As_U.Tus ("PRK"),
            As_U.Tus ("North Korea"
                    & " (Democratic People's Republic of Korea)"
                    & " (Korea, North)")),
    ("148", As_U.Tus ("NIC"),
            As_U.Tus ("Nicaragua (Republic of _)")),
    ("149", As_U.Tus ("GRC"),
            As_U.Tus ("Greece (Hellenic Republic) (Hellas)")),
    ("150", As_U.Tus ("TJK"),
            As_U.Tus ("Tajikistan (Republic of _)")),
    ("151", As_U.Tus ("BGD"),
            As_U.Tus ("Bangladesh (People's Republic of _)")),
    ("152", As_U.Tus ("NPL"),
            As_U.Tus ("Nepal (Federal Democratic Republic of _)")),
    ("153", As_U.Tus ("TUN"),
            As_U.Tus ("Tunisia (Tunisian Republic) (Republic of _)")),
    ("154", As_U.Tus ("SUR"),
            As_U.Tus ("Suriname (Republic of _)")),
    ("155", As_U.Tus ("URY"),
            As_U.Tus ("Uruguay (Eastern Republic of _)")),
    ("156", As_U.Tus ("KHM"),
            As_U.Tus ("Cambodia (Kingdom of _)")),
    ("157", As_U.Tus ("SYR"),
            As_U.Tus ("Syria (Syrian Arab Republic)")),
    ("158", As_U.Tus ("SEN"),
            As_U.Tus ("Senegal (Republic of _)")),
    ("159", As_U.Tus ("KGZ"),
            As_U.Tus ("Kyrgyzstan (Kyrgyz Republic)")),
    ("160", As_U.Tus ("BLR"),
            As_U.Tus ("Belarus (Republic of _)")),
    ("161", As_U.Tus ("GUY"),
            As_U.Tus ("Guyana (Co-operative Republic of _)")),
    ("162", As_U.Tus ("LAO"),
            As_U.Tus ("Laos (Lao People's Democratic Republic)")),
    ("163", As_U.Tus ("ROU"),
            As_U.Tus ("Romania")),
    ("164", As_U.Tus ("GHA"),
            As_U.Tus ("Ghana (Republic of _)")),
    ("165", As_U.Tus ("UGA"),
            As_U.Tus ("Uganda (Republic of _)")),
    ("166", As_U.Tus ("GBR"),
            As_U.Tus ("United Kingdom"
                    & " (Scotland) (Great Britain) (England)"
                    & " (Northern Ireland) (Ireland, Northern) (Britain)"
                    & " (United Kingdom of Great Britain and Northern"
                    & " Ireland)")),
    ("167", As_U.Tus ("GIN"),
            As_U.Tus ("Guinea (Republic of _) (Guinea-Conakry)")),
    ("168", As_U.Tus ("ECU"),
            As_U.Tus ("Ecuador (Republic of _)")),
    ("169", As_U.Tus ("ESH"),
            As_U.Tus ("Western Sahara (Sahrawi Arab Democratic Republic)")),
    ("170", As_U.Tus ("GAB"),
            As_U.Tus ("Gabon (Gabonese Republic)")),
    ("171", As_U.Tus ("NZL"),
            As_U.Tus ("New Zealand")),
    ("172", As_U.Tus ("BFA"),
            As_U.Tus ("Burkina Faso")),
    ("173", As_U.Tus ("PHL"),
            As_U.Tus ("Philippines (Republic of the _)")),
    ("174", As_U.Tus ("ITA"),
            As_U.Tus ("Italy (Italian Republic)")),
    ("175", As_U.Tus ("OMN"),
            As_U.Tus ("Oman (Sultanate of _)")),
    ("176", As_U.Tus ("POL"),
            As_U.Tus ("Poland (Republic of _)")),
    ("177", As_U.Tus ("CIV"),
            As_U.Tus ("Ivory Coast"
                    & " (Cote d'Ivoire) (Republic of Cote d'Ivoire)")),
    ("178", As_U.Tus ("NOR"),
            As_U.Tus ("Norway (Kingdom of _)")),
    ("179", As_U.Tus ("MYS"),
            As_U.Tus ("Malaysia")),
    ("180", As_U.Tus ("VNM"),
            As_U.Tus ("Vietnam (Socialist Republic of _)")),
    ("181", As_U.Tus ("FIN"),
            As_U.Tus ("Finland (Republic of _)")),
    ("182", As_U.Tus ("COG"),
            As_U.Tus ("Congo-Brazzaville"
                    & " (West Congo) (Republic of the Congo)")),
    ("183", As_U.Tus ("DEU"),
            As_U.Tus ("Germany (Federal Republic of _)")),
    ("184", As_U.Tus ("JPN"),
            As_U.Tus ("Japan")),
    ("185", As_U.Tus ("ZWE"),
            As_U.Tus ("Zimbabwe (Republic of _)")),
    ("186", As_U.Tus ("PRY"),
            As_U.Tus ("Paraguay (Republic of _)")),
    ("187", As_U.Tus ("IRQ"),
            As_U.Tus ("Iraq (Republic of _)")),
    ("188", As_U.Tus ("MAR"),
            As_U.Tus ("Morocco (Kingdom of _)")),
    ("189", As_U.Tus ("UZB"),
            As_U.Tus ("Uzbekistan (Republic of _)")),
    ("190", As_U.Tus ("SWE"),
            As_U.Tus ("Sweden (Kingdom of _)")),
    ("191", As_U.Tus ("PNG"),
            As_U.Tus ("Papua New Guinea (Independent State of _)")),
    ("192", As_U.Tus ("CMR"),
            As_U.Tus ("Cameroon (Republic of _)")),
    ("193", As_U.Tus ("TKM"),
            As_U.Tus ("Turkmenistan")),
    ("194", As_U.Tus ("ESP"),
            As_U.Tus ("Spain (Kingdom of _)")),
    ("195", As_U.Tus ("THA"),
            As_U.Tus ("Thailand (Kingdom of _)")),
    ("196", As_U.Tus ("YEM"),
            As_U.Tus ("Yemen (Republic of _)")),
    ("197", As_U.Tus ("FRA"),
            As_U.Tus ("France (French Republic)")),
    ("198", As_U.Tus ("ALA"),
            As_U.Tus ("Aaland Islands")),
    ("199", As_U.Tus ("KEN"),
            As_U.Tus ("Kenya (Republic of _)")),
    ("200", As_U.Tus ("BWA"),
            As_U.Tus ("Botswana (Republic of _)")),
    ("201", As_U.Tus ("MDG"),
            As_U.Tus ("Madagascar (Republic of _)")),
    ("202", As_U.Tus ("UKR"),
            As_U.Tus ("Ukraine")),
    ("203", As_U.Tus ("SSD"),
            As_U.Tus ("South Sudan (Republic of _)")),
    ("204", As_U.Tus ("CAF"),
            As_U.Tus ("Central African Republic")),
    ("205", As_U.Tus ("SOM"),
            As_U.Tus ("Somalia (Federal Republic of _)")),
    ("206", As_U.Tus ("AFG"),
            As_U.Tus ("Afghanistan (Islamic Republic of _)")),
    ("207", As_U.Tus ("MMR"),
            As_U.Tus ("Myanmar (Republic of the Union of _) (Burma)")),
    ("208", As_U.Tus ("ZMB"),
            As_U.Tus ("Zambia (Republic of _)")),
    ("209", As_U.Tus ("CHL"),
            As_U.Tus ("Chile (Republic of _)")),
    ("210", As_U.Tus ("TUR"),
            As_U.Tus ("Turkey (Republic of _)")),
    ("211", As_U.Tus ("PAK"),
            As_U.Tus ("Pakistan (Islamic Republic of _)")),
    ("212", As_U.Tus ("MOZ"),
            As_U.Tus ("Mozambique (Republic of _)")),
    ("213", As_U.Tus ("NAM"),
            As_U.Tus ("Namibia (Republic of _)")),
    ("214", As_U.Tus ("VEN"),
            As_U.Tus ("Venezuela (Bolivarian Republic of _)")),
    ("215", As_U.Tus ("NGA"),
            As_U.Tus ("Nigeria (Federal Republic of _)")),
    ("216", As_U.Tus ("TZA"),
            As_U.Tus ("Tanzania (United Republic of _)")),
    ("217", As_U.Tus ("EGY"),
            As_U.Tus ("Egypt (Arab Republic of _)")),
    ("218", As_U.Tus ("MRT"),
            As_U.Tus ("Mauritania (Islamic Republic of _)")),
    ("219", As_U.Tus ("BOL"),
            As_U.Tus ("Bolivia (Plurinational State of _)")),
    ("220", As_U.Tus ("ETH"),
            As_U.Tus ("Ethiopia (Federal Democratic Republic of _)")),
    ("221", As_U.Tus ("COL"),
            As_U.Tus ("Colombia (Republic of _)")),
    ("222", As_U.Tus ("ZAF"),
            As_U.Tus ("South Africa (Republic of _)")),
    ("223", As_U.Tus ("MLI"),
            As_U.Tus ("Mali (Republic of _)")),
    ("224", As_U.Tus ("AGO"),
            As_U.Tus ("Angola (Republic of _)")),
    ("225", As_U.Tus ("NER"),
            As_U.Tus ("Niger (Republic of _)")),
    ("226", As_U.Tus ("TCD"),
            As_U.Tus ("Chad (Republic of _)")),
    ("227", As_U.Tus ("PER"),
            As_U.Tus ("Peru (Republic of _)")),
    ("228", As_U.Tus ("MNG"),
            As_U.Tus ("Mongolia")),
    ("229", As_U.Tus ("IRN"),
            As_U.Tus ("Iran (Persia) (Islamic Republic of _)")),
    ("230", As_U.Tus ("LBY"),
            As_U.Tus ("Libya")),
    ("231", As_U.Tus ("SDN"),
            As_U.Tus ("Sudan (Republic of the _)")),
    ("232", As_U.Tus ("IDN"),
            As_U.Tus ("Indonesia (Republic of _)")),
    ("233", As_U.Tus ("MX-DIF"),
            As_U.Tus ("Federal District")),
    ("234", As_U.Tus ("MX-TLA"),
            As_U.Tus ("Tlaxcala")),
    ("235", As_U.Tus ("MX-MOR"),
            As_U.Tus ("Morelos")),
    ("236", As_U.Tus ("MX-AGU"),
            As_U.Tus ("Aguascalientes")),
    ("237", As_U.Tus ("MX-CL"),
            As_U.Tus ("Colima")),
    ("238", As_U.Tus ("MX-QUE"),
            As_U.Tus ("Queretaro")),
    ("239", As_U.Tus ("MX-HID"),
            As_U.Tus ("Hidalgo")),
    ("240", As_U.Tus ("MX-MX"),
            As_U.Tus ("Mexico State")),
    ("241", As_U.Tus ("MX-TAB"),
            As_U.Tus ("Tabasco")),
    ("242", As_U.Tus ("MX-NAY"),
            As_U.Tus ("Nayarit")),
    ("243", As_U.Tus ("MX-GUA"),
            As_U.Tus ("Guanajuato")),
    ("244", As_U.Tus ("MX-PUE"),
            As_U.Tus ("Puebla")),
    ("245", As_U.Tus ("MX-YUC"),
            As_U.Tus ("Yucatan")),
    ("246", As_U.Tus ("MX-ROO"),
            As_U.Tus ("Quintana Roo")),
    ("247", As_U.Tus ("MX-SIN"),
            As_U.Tus ("Sinaloa")),
    ("248", As_U.Tus ("MX-CAM"),
            As_U.Tus ("Campeche")),
    ("249", As_U.Tus ("MX-MIC"),
            As_U.Tus ("Michoacan")),
    ("250", As_U.Tus ("MX-SLP"),
            As_U.Tus ("San Luis Potosi")),
    ("251", As_U.Tus ("MX-GRO"),
            As_U.Tus ("Guerrero")),
    ("252", As_U.Tus ("MX-NLE"),
            As_U.Tus ("Nuevo Leon (New Leon)")),
    ("253", As_U.Tus ("MX-BCN"),
            As_U.Tus ("Baja California")),
    ("254", As_U.Tus ("MX-VER"),
            As_U.Tus ("Veracruz")),
    ("255", As_U.Tus ("MX-CHP"),
            As_U.Tus ("Chiapas")),
    ("256", As_U.Tus ("MX-BCS"),
            As_U.Tus ("Baja California Sur")),
    ("257", As_U.Tus ("MX-ZAC"),
            As_U.Tus ("Zacatecas")),
    ("258", As_U.Tus ("MX-JAL"),
            As_U.Tus ("Jalisco")),
    ("259", As_U.Tus ("MX-TAM"),
            As_U.Tus ("Tamaulipas")),
    ("260", As_U.Tus ("MX-OAX"),
            As_U.Tus ("Oaxaca")),
    ("261", As_U.Tus ("MX-DUR"),
            As_U.Tus ("Durango")),
    ("262", As_U.Tus ("MX-COA"),
            As_U.Tus ("Coahuila")),
    ("263", As_U.Tus ("MX-SON"),
            As_U.Tus ("Sonora")),
    ("264", As_U.Tus ("MX-CHH"),
            As_U.Tus ("Chihuahua")),
    ("265", As_U.Tus ("GRL"),
            As_U.Tus ("Greenland")),
    ("266", As_U.Tus ("SAU"),
            As_U.Tus ("Saudi Arabia (Kingdom of _)")),
    ("267", As_U.Tus ("COD"),
            As_U.Tus ("Congo-Kinshasa"
                    & " (Democratic Republic of the Congo) (East Congo)")),
    ("268", As_U.Tus ("DZA"),
            As_U.Tus ("Algeria (People's Democratic Republic of _)")),
    ("269", As_U.Tus ("KAZ"),
            As_U.Tus ("Kazakhstan (Republic of _)")),
    ("270", As_U.Tus ("ARG"),
            As_U.Tus ("Argentina (Argentine Republic)")),
    ("271", As_U.Tus ("IN-DD"),
            As_U.Tus ("Daman and Diu")),
    ("272", As_U.Tus ("IN-DN"),
            As_U.Tus ("Dadra and Nagar Haveli (Dadra) (Nagar Haveli)")),
    ("273", As_U.Tus ("IN-CH"),
            As_U.Tus ("Chandigarh")),
    ("274", As_U.Tus ("IN-AN"),
            As_U.Tus ("Andaman and Nicobar (Andaman) (Nicobar)")),
    ("275", As_U.Tus ("IN-LD"),
            As_U.Tus ("Lakshadweep")),
    ("276", As_U.Tus ("IN-DL"),
            As_U.Tus ("Delhi (National Capital Territory of _)")),
    ("277", As_U.Tus ("IN-ML"),
            As_U.Tus ("Meghalaya")),
    ("278", As_U.Tus ("IN-NL"),
            As_U.Tus ("Nagaland")),
    ("279", As_U.Tus ("IN-MN"),
            As_U.Tus ("Manipur")),
    ("280", As_U.Tus ("IN-TR"),
            As_U.Tus ("Tripura")),
    ("281", As_U.Tus ("IN-MZ"),
            As_U.Tus ("Mizoram")),
    ("282", As_U.Tus ("IN-SK"),
            As_U.Tus ("Sikkim")),
    ("283", As_U.Tus ("IN-PB"),
            As_U.Tus ("Punjab")),
    ("284", As_U.Tus ("IN-HR"),
            As_U.Tus ("Haryana")),
    ("285", As_U.Tus ("IN-AR"),
            As_U.Tus ("Arunachal Pradesh")),
    ("286", As_U.Tus ("IN-AS"),
            As_U.Tus ("Assam")),
    ("287", As_U.Tus ("IN-BR"),
            As_U.Tus ("Bihar")),
    ("288", As_U.Tus ("IN-UT"),
            As_U.Tus ("Uttarakhand")),
    ("289", As_U.Tus ("IN-GA"),
            As_U.Tus ("Goa")),
    ("290", As_U.Tus ("IN-KL"),
            As_U.Tus ("Kerala")),
    ("291", As_U.Tus ("IN-TN"),
            As_U.Tus ("Tamil Nadu")),
    ("292", As_U.Tus ("IN-HP"),
            As_U.Tus ("Himachal Pradesh")),
    ("293", As_U.Tus ("IN-JK"),
            As_U.Tus ("Jammu and Kashmir (Jammu) (Kashmir)")),
    ("294", As_U.Tus ("IN-CT"),
            As_U.Tus ("Chhattisgarh")),
    ("295", As_U.Tus ("IN-JH"),
            As_U.Tus ("Jharkhand")),
    ("296", As_U.Tus ("IN-KA"),
            As_U.Tus ("Karnataka")),
    ("297", As_U.Tus ("IN-RJ"),
            As_U.Tus ("Rajasthan")),
    ("298", As_U.Tus ("IN-OR"),
            As_U.Tus ("Odisha (Orissa)")),
    ("299", As_U.Tus ("IN-GJ"),
            As_U.Tus ("Gujarat")),
    ("300", As_U.Tus ("IN-WB"),
            As_U.Tus ("West Bengal")),
    ("301", As_U.Tus ("IN-MP"),
            As_U.Tus ("Madhya Pradesh")),
    ("302", As_U.Tus ("IN-TG"),
            As_U.Tus ("Telangana")),
    ("303", As_U.Tus ("IN-AP"),
            As_U.Tus ("Andhra Pradesh")),
    ("304", As_U.Tus ("IN-MH"),
            As_U.Tus ("Maharashtra")),
    ("305", As_U.Tus ("IN-UP"),
            As_U.Tus ("Uttar Pradesh")),
    ("306", As_U.Tus ("IN-PY"),
            As_U.Tus ("Puducherry")),
    ("307", As_U.Tus ("AU-NSW"),
            As_U.Tus ("New South Wales")),
    ("308", As_U.Tus ("AU-ACT"),
            As_U.Tus ("Australian Capital Territory")),
    ("309", As_U.Tus ("AU-JBT"),
            As_U.Tus ("Jervis Bay Territory")),
    ("310", As_U.Tus ("AU-NT"),
            As_U.Tus ("Northern Territory")),
    ("311", As_U.Tus ("AU-SA"),
            As_U.Tus ("South Australia")),
    ("312", As_U.Tus ("AU-TAS"),
            As_U.Tus ("Tasmania")),
    ("313", As_U.Tus ("AU-VIC"),
            As_U.Tus ("Victoria")),
    ("314", As_U.Tus ("AU-WA"),
            As_U.Tus ("Western Australia")),
    ("315", As_U.Tus ("AU-QLD"),
            As_U.Tus ("Queensland")),
    ("316", As_U.Tus ("BR-DF"),
            As_U.Tus ("Distrito Federal")),
    ("317", As_U.Tus ("BR-SE"),
            As_U.Tus ("Sergipe")),
    ("318", As_U.Tus ("BR-AL"),
            As_U.Tus ("Alagoas")),
    ("319", As_U.Tus ("BR-RJ"),
            As_U.Tus ("Rio de Janeiro")),
    ("320", As_U.Tus ("BR-ES"),
            As_U.Tus ("Espirito Santo")),
    ("321", As_U.Tus ("BR-RN"),
            As_U.Tus ("Rio Grande do Norte")),
    ("322", As_U.Tus ("BR-PB"),
            As_U.Tus ("Paraiba")),
    ("323", As_U.Tus ("BR-SC"),
            As_U.Tus ("Santa Catarina")),
    ("324", As_U.Tus ("BR-PE"),
            As_U.Tus ("Pernambuco")),
    ("325", As_U.Tus ("BR-AP"),
            As_U.Tus ("Amapa")),
    ("326", As_U.Tus ("BR-CE"),
            As_U.Tus ("Ceara")),
    ("327", As_U.Tus ("BR-AC"),
            As_U.Tus ("Acre")),
    ("328", As_U.Tus ("BR-PR"),
            As_U.Tus ("Parana")),
    ("329", As_U.Tus ("BR-RR"),
            As_U.Tus ("Roraima")),
    ("330", As_U.Tus ("BR-RO"),
            As_U.Tus ("Rondonia")),
    ("331", As_U.Tus ("BR-SP"),
            As_U.Tus ("Sao Paulo")),
    ("332", As_U.Tus ("BR-PI"),
            As_U.Tus ("Piaui")),
    ("333", As_U.Tus ("BR-TO"),
            As_U.Tus ("Tocantins")),
    ("334", As_U.Tus ("BR-RS"),
            As_U.Tus ("Rio Grande do Sul")),
    ("335", As_U.Tus ("BR-MA"),
            As_U.Tus ("Maranhao")),
    ("336", As_U.Tus ("BR-GO"),
            As_U.Tus ("Goias")),
    ("337", As_U.Tus ("BR-MS"),
            As_U.Tus ("Mato Grosso do Sul")),
    ("338", As_U.Tus ("BR-BA"),
            As_U.Tus ("Bahia")),
    ("339", As_U.Tus ("BR-MG"),
            As_U.Tus ("Minas Gerais")),
    ("340", As_U.Tus ("BR-MT"),
            As_U.Tus ("Mato Grosso")),
    ("341", As_U.Tus ("BR-PA"),
            As_U.Tus ("Para")),
    ("342", As_U.Tus ("BR-AM"),
            As_U.Tus ("Amazonas")),
    ("343", As_U.Tus ("US-DC"),
            As_U.Tus ("District of Columbia (Washington, D.C.)")),
    ("344", As_U.Tus ("US-RI"),
            As_U.Tus ("Rhode Island")),
    ("345", As_U.Tus ("US-DE"),
            As_U.Tus ("Delaware")),
    ("346", As_U.Tus ("US-CT"),
            As_U.Tus ("Connecticut")),
    ("347", As_U.Tus ("US-NJ"),
            As_U.Tus ("New Jersey")),
    ("348", As_U.Tus ("US-NH"),
            As_U.Tus ("New Hampshire")),
    ("349", As_U.Tus ("US-VT"),
            As_U.Tus ("Vermont")),
    ("350", As_U.Tus ("US-MA"),
            As_U.Tus ("Massachusetts (Commonwealth of _)")),
    ("351", As_U.Tus ("US-HI"),
            As_U.Tus ("Hawaii")),
    ("352", As_U.Tus ("US-MD"),
            As_U.Tus ("Maryland")),
    ("353", As_U.Tus ("US-WV"),
            As_U.Tus ("West Virginia")),
    ("354", As_U.Tus ("US-SC"),
            As_U.Tus ("South Carolina")),
    ("355", As_U.Tus ("US-ME"),
            As_U.Tus ("Maine")),
    ("356", As_U.Tus ("US-IN"),
            As_U.Tus ("Indiana")),
    ("357", As_U.Tus ("US-KY"),
            As_U.Tus ("Kentucky (Commonwealth of _)")),
    ("358", As_U.Tus ("US-TN"),
            As_U.Tus ("Tennessee")),
    ("359", As_U.Tus ("US-VA"),
            As_U.Tus ("Virginia (Commonwealth of _)")),
    ("360", As_U.Tus ("US-OH"),
            As_U.Tus ("Ohio")),
    ("361", As_U.Tus ("US-PA"),
            As_U.Tus ("Pennsylvania (Commonwealth of _)")),
    ("362", As_U.Tus ("US-MS"),
            As_U.Tus ("Mississippi")),
    ("363", As_U.Tus ("US-LA"),
            As_U.Tus ("Louisiana")),
    ("364", As_U.Tus ("US-AL"),
            As_U.Tus ("Alabama")),
    ("365", As_U.Tus ("US-AR"),
            As_U.Tus ("Arkansas")),
    ("366", As_U.Tus ("US-NC"),
            As_U.Tus ("North Carolina")),
    ("367", As_U.Tus ("US-NY"),
            As_U.Tus ("New York")),
    ("368", As_U.Tus ("US-IA"),
            As_U.Tus ("Iowa")),
    ("369", As_U.Tus ("US-IL"),
            As_U.Tus ("Illinois")),
    ("370", As_U.Tus ("US-GA"),
            As_U.Tus ("Georgia")),
    ("371", As_U.Tus ("US-WI"),
            As_U.Tus ("Wisconsin")),
    ("372", As_U.Tus ("US-FL"),
            As_U.Tus ("Florida")),
    ("373", As_U.Tus ("US-MO"),
            As_U.Tus ("Missouri")),
    ("374", As_U.Tus ("US-OK"),
            As_U.Tus ("Oklahoma")),
    ("375", As_U.Tus ("US-ND"),
            As_U.Tus ("North Dakota")),
    ("376", As_U.Tus ("US-WA"),
            As_U.Tus ("Washington")),
    ("377", As_U.Tus ("US-SD"),
            As_U.Tus ("South Dakota")),
    ("378", As_U.Tus ("US-NE"),
            As_U.Tus ("Nebraska")),
    ("379", As_U.Tus ("US-KS"),
            As_U.Tus ("Kansas")),
    ("380", As_U.Tus ("US-ID"),
            As_U.Tus ("Idaho")),
    ("381", As_U.Tus ("US-UT"),
            As_U.Tus ("Utah")),
    ("382", As_U.Tus ("US-MN"),
            As_U.Tus ("Minnesota")),
    ("383", As_U.Tus ("US-MI"),
            As_U.Tus ("Michigan")),
    ("384", As_U.Tus ("US-WY"),
            As_U.Tus ("Wyoming")),
    ("385", As_U.Tus ("US-OR"),
            As_U.Tus ("Oregon")),
    ("386", As_U.Tus ("US-CO"),
            As_U.Tus ("Colorado")),
    ("387", As_U.Tus ("US-NV"),
            As_U.Tus ("Nevada")),
    ("388", As_U.Tus ("US-AZ"),
            As_U.Tus ("Arizona")),
    ("389", As_U.Tus ("US-NM"),
            As_U.Tus ("New Mexico")),
    ("390", As_U.Tus ("US-MT"),
            As_U.Tus ("Montana")),
    ("391", As_U.Tus ("US-CA"),
            As_U.Tus ("California")),
    ("392", As_U.Tus ("US-TX"),
            As_U.Tus ("Texas")),
    ("393", As_U.Tus ("US-AK"),
            As_U.Tus ("Alaska")),
    ("394", As_U.Tus ("CA-BC"),
            As_U.Tus ("British Columbia")),
    ("395", As_U.Tus ("CA-AB"),
            As_U.Tus ("Alberta")),
    ("396", As_U.Tus ("CA-ON"),
            As_U.Tus ("Ontario")),
    ("397", As_U.Tus ("CA-QC"),
            As_U.Tus ("Quebec")),
    ("398", As_U.Tus ("CA-SK"),
            As_U.Tus ("Saskatchewan")),
    ("399", As_U.Tus ("CA-MB"),
            As_U.Tus ("Manitoba")),
    ("400", As_U.Tus ("CA-NL"),
            As_U.Tus ("Newfoundland and Labrador (Newfoundland) (Labrador)")),
    ("401", As_U.Tus ("CA-NB"),
            As_U.Tus ("New Brunswick")),
    ("402", As_U.Tus ("CA-NS"),
            As_U.Tus ("Nova Scotia")),
    ("403", As_U.Tus ("CA-PE"),
            As_U.Tus ("Prince Edward Island")),
    ("404", As_U.Tus ("CA-YT"),
            As_U.Tus ("Yukon")),
    ("405", As_U.Tus ("CA-NT"),
            As_U.Tus ("Northwest Territories")),
    ("406", As_U.Tus ("CA-NU"),
            As_U.Tus ("Nunavut")),
    ("407", As_U.Tus ("IND"),
            As_U.Tus ("India (Republic of _)")),
    ("408", As_U.Tus ("AUS"),
            As_U.Tus ("Australia (Commonwealth of _)")),
    ("409", As_U.Tus ("BRA"),
            As_U.Tus ("Brazil (Federative Republic of _)")),
    ("410", As_U.Tus ("USA"),
            As_U.Tus ("USA (United States of America) (America)")),
    ("411", As_U.Tus ("MEX"),
            As_U.Tus ("Mexico (United Mexican States)")),
    ("412", As_U.Tus ("RU-MOW"),
            As_U.Tus ("Moscow")),
    ("413", As_U.Tus ("RU-SPE"),
            As_U.Tus ("Saint Petersburg")),
    ("414", As_U.Tus ("RU-KGD"),
            As_U.Tus ("Kaliningrad Oblast")),
    ("415", As_U.Tus ("RU-IN"),
            As_U.Tus ("Ingushetia")),
    ("416", As_U.Tus ("RU-AD"),
            As_U.Tus ("Adygea Republic")),
    ("417", As_U.Tus ("RU-SE"),
            As_U.Tus ("North Ossetia-Alania Republic")),
    ("418", As_U.Tus ("RU-KB"),
            As_U.Tus ("Kabardino-Balkar Republic")),
    ("419", As_U.Tus ("RU-KC"),
            As_U.Tus ("Karachay-Cherkess Republic")),
    ("420", As_U.Tus ("RU-CE"),
            As_U.Tus ("Chechen Republic (Chechnya) (Ichkeria)")),
    ("421", As_U.Tus ("RU-CU"),
            As_U.Tus ("Chuvash Republic")),
    ("422", As_U.Tus ("RU-IVA"),
            As_U.Tus ("Ivanovo Oblast")),
    ("423", As_U.Tus ("RU-LIP"),
            As_U.Tus ("Lipetsk Oblast")),
    ("424", As_U.Tus ("RU-ORL"),
            As_U.Tus ("Oryol Oblast")),
    ("425", As_U.Tus ("RU-TUL"),
            As_U.Tus ("Tula Oblast")),
    ("426", As_U.Tus ("RU-BE"),
            As_U.Tus ("Belgorod Oblast")),
    ("427", As_U.Tus ("RU-VLA"),
            As_U.Tus ("Vladimir Oblast")),
    ("428", As_U.Tus ("RU-KRS"),
            As_U.Tus ("Kursk Oblast")),
    ("429", As_U.Tus ("RU-KLU"),
            As_U.Tus ("Kaluga Oblast")),
    ("430", As_U.Tus ("RU-TT"),
            As_U.Tus ("Tambov Oblast")),
    ("431", As_U.Tus ("RU-BRY"),
            As_U.Tus ("Bryansk Oblast")),
    ("432", As_U.Tus ("RU-YAR"),
            As_U.Tus ("Yaroslavl Oblast")),
    ("433", As_U.Tus ("RU-RYA"),
            As_U.Tus ("Ryazan Oblast")),
    ("434", As_U.Tus ("RU-AST"),
            As_U.Tus ("Astrakhan Oblast")),
    ("435", As_U.Tus ("RU-MOS"),
            As_U.Tus ("Moscow Oblast")),
    ("436", As_U.Tus ("RU-SMO"),
            As_U.Tus ("Smolensk Oblast")),
    ("437", As_U.Tus ("RU-DA"),
            As_U.Tus ("Dagestan Republic")),
    ("438", As_U.Tus ("RU-VOR"),
            As_U.Tus ("Voronezh Oblast")),
    ("439", As_U.Tus ("RU-NGR"),
            As_U.Tus ("Novgorod Oblast")),
    ("440", As_U.Tus ("RU-PSK"),
            As_U.Tus ("Pskov Oblast")),
    ("441", As_U.Tus ("RU-KOS"),
            As_U.Tus ("Kostroma Oblast")),
    ("442", As_U.Tus ("RU-STA"),
            As_U.Tus ("Stavropol Krai")),
    ("443", As_U.Tus ("RU-KDA"),
            As_U.Tus ("Krasnodar Krai")),
    ("444", As_U.Tus ("RU-KL"),
            As_U.Tus ("Kalmykia Republic")),
    ("445", As_U.Tus ("RU-TVE"),
            As_U.Tus ("Tver Oblast")),
    ("446", As_U.Tus ("RU-LEN"),
            As_U.Tus ("Leningrad Oblast")),
    ("447", As_U.Tus ("RU-ROS"),
            As_U.Tus ("Rostov Oblast")),
    ("448", As_U.Tus ("RU-VGG"),
            As_U.Tus ("Volgograd Oblast")),
    ("449", As_U.Tus ("RU-VLG"),
            As_U.Tus ("Vologda Oblast")),
    ("450", As_U.Tus ("RU-MUR"),
            As_U.Tus ("Murmansk Oblast")),
    ("451", As_U.Tus ("RU-KR"),
            As_U.Tus ("Karelia Republic")),
    ("452", As_U.Tus ("RU-NEN"),
            As_U.Tus ("Nenets Autonomous Okrug")),
    ("453", As_U.Tus ("RU-KO"),
            As_U.Tus ("Komi Republic")),
    ("454", As_U.Tus ("RU-ARK"),
            As_U.Tus ("Arkhangelsk Oblast")),
    ("455", As_U.Tus ("RU-MO"),
            As_U.Tus ("Mordovia Republic")),
    ("456", As_U.Tus ("RU-NIZ"),
            As_U.Tus ("Nizhny Novgorod Oblast")),
    ("457", As_U.Tus ("RU-PNZ"),
            As_U.Tus ("Penza Oblast")),
    ("458", As_U.Tus ("RU-KI"),
            As_U.Tus ("Kirov Oblast")),
    ("459", As_U.Tus ("RU-ME"),
            As_U.Tus ("Mari El Republic")),
    ("460", As_U.Tus ("RU-ORE"),
            As_U.Tus ("Orenburg Oblast")),
    ("461", As_U.Tus ("RU-ULY"),
            As_U.Tus ("Ulyanovsk Oblast")),
    ("462", As_U.Tus ("RU-PM"),
            As_U.Tus ("Perm Krai")),
    ("463", As_U.Tus ("RU-BA"),
            As_U.Tus ("Bashkortostan Republic")),
    ("464", As_U.Tus ("RU-UD"),
            As_U.Tus ("Udmurt Republic")),
    ("465", As_U.Tus ("RU-TA"),
            As_U.Tus ("Tatarstan Republic")),
    ("466", As_U.Tus ("RU-SAM"),
            As_U.Tus ("Samara Oblast")),
    ("467", As_U.Tus ("RU-SAR"),
            As_U.Tus ("Saratov Oblast")),
    ("468", As_U.Tus ("RU-YAN"),
            As_U.Tus ("Yamalo-Nenets")),
    ("469", As_U.Tus ("RU-KM"),
            As_U.Tus ("Khanty-Mansi")),
    ("470", As_U.Tus ("RU-SVE"),
            As_U.Tus ("Sverdlovsk Oblast")),
    ("471", As_U.Tus ("RU-TYU"),
            As_U.Tus ("Tyumen Oblast")),
    ("472", As_U.Tus ("RU-KGN"),
            As_U.Tus ("Kurgan Oblast")),
    ("473", As_U.Tus ("RU-CH"),
            As_U.Tus ("Chelyabinsk Oblast")),
    ("474", As_U.Tus ("RU-BU"),
            As_U.Tus ("Buryatia Republic")),
    ("475", As_U.Tus ("RU-ZAB"),
            As_U.Tus ("Zabaykalsky Krai")),
    ("476", As_U.Tus ("RU-IRK"),
            As_U.Tus ("Irkutsk Oblast")),
    ("477", As_U.Tus ("RU-NVS"),
            As_U.Tus ("Novosibirsk Oblast")),
    ("478", As_U.Tus ("RU-TOM"),
            As_U.Tus ("Tomsk Oblast")),
    ("479", As_U.Tus ("RU-OMS"),
            As_U.Tus ("Omsk Oblast")),
    ("480", As_U.Tus ("RU-KK"),
            As_U.Tus ("Khakassia Republic")),
    ("481", As_U.Tus ("RU-KEM"),
            As_U.Tus ("Kemerovo Oblast")),
    ("482", As_U.Tus ("RU-AL"),
            As_U.Tus ("Altai Republic")),
    ("483", As_U.Tus ("RU-ALT"),
            As_U.Tus ("Altai Krai")),
    ("484", As_U.Tus ("RU-TY"),
            As_U.Tus ("Tuva Republic")),
    ("485", As_U.Tus ("RU-KYA"),
            As_U.Tus ("Krasnoyarsk Krai")),
    ("486", As_U.Tus ("RU-MAG"),
            As_U.Tus ("Magadan Oblast")),
    ("487", As_U.Tus ("RU-CHU"),
            As_U.Tus ("Chukotka Okrug")),
    ("488", As_U.Tus ("RU-KAM"),
            As_U.Tus ("Kamchatka Krai")),
    ("489", As_U.Tus ("RU-SAK"),
            As_U.Tus ("Sakhalin Oblast")),
    ("490", As_U.Tus ("RU-PO"),
            As_U.Tus ("Primorsky Krai")),
    ("491", As_U.Tus ("RU-YEV"),
            As_U.Tus ("Jewish Autonomous Oblast")),
    ("492", As_U.Tus ("RU-KHA"),
            As_U.Tus ("Khabarovsk Krai")),
    ("493", As_U.Tus ("RU-AMU"),
            As_U.Tus ("Amur Oblast")),
    ("494", As_U.Tus ("RU-SA"),
            As_U.Tus ("Sakha Republic (Yakutia Republic)")),
    ("495", As_U.Tus ("CAN"),
            As_U.Tus ("Canada")),
    ("496", As_U.Tus ("RUS"),
            As_U.Tus ("Russia (Russian Federation)")),
    ("497", As_U.Tus ("CN-SH"),
            As_U.Tus ("Shanghai Municipality")),
    ("498", As_U.Tus ("CN-TJ"),
            As_U.Tus ("Tianjin Municipality")),
    ("499", As_U.Tus ("CN-BJ"),
            As_U.Tus ("Beijing Municipality")),
    ("500", As_U.Tus ("CN-HI"),
            As_U.Tus ("Hainan Province")),
    ("501", As_U.Tus ("CN-NX"),
            As_U.Tus ("Ningxia Hui Autonomous Region")),
    ("502", As_U.Tus ("CN-CQ"),
            As_U.Tus ("Chongqing Municipality")),
    ("503", As_U.Tus ("CN-ZJ"),
            As_U.Tus ("Zhejiang Province")),
    ("504", As_U.Tus ("CN-JS"),
            As_U.Tus ("Jiangsu Province")),
    ("505", As_U.Tus ("CN-FJ"),
            As_U.Tus ("Fujian Province")),
    ("506", As_U.Tus ("CN-AH"),
            As_U.Tus ("Anhui Province")),
    ("507", As_U.Tus ("CN-LN"),
            As_U.Tus ("Liaoning Province")),
    ("508", As_U.Tus ("CN-SD"),
            As_U.Tus ("Shandong Province")),
    ("509", As_U.Tus ("CN-SX"),
            As_U.Tus ("Shanxi Province")),
    ("510", As_U.Tus ("CN-JX"),
            As_U.Tus ("Jiangxi Province")),
    ("511", As_U.Tus ("CN-HA"),
            As_U.Tus ("Henan Province")),
    ("512", As_U.Tus ("CN-GZ"),
            As_U.Tus ("Guizhou Province")),
    ("513", As_U.Tus ("CN-GD"),
            As_U.Tus ("Guangdong Province")),
    ("514", As_U.Tus ("CN-HB"),
            As_U.Tus ("Hubei Province")),
    ("515", As_U.Tus ("CN-JL"),
            As_U.Tus ("Jilin Province")),
    ("516", As_U.Tus ("CN-HE"),
            As_U.Tus ("Hebei Province (Yanzhao Province)")),
    ("517", As_U.Tus ("CN-SN"),
            As_U.Tus ("Shaanxi Province")),
    ("518", As_U.Tus ("CN-NM"),
            As_U.Tus ("Nei Mongol Autonomous Region (Inner Mongolia)")),
    ("519", As_U.Tus ("CN-HL"),
            As_U.Tus ("Heilongjiang Province")),
    ("520", As_U.Tus ("CN-HN"),
            As_U.Tus ("Hunan Province")),
    ("521", As_U.Tus ("CN-GX"),
            As_U.Tus ("Guangxi Zhuang Autonomous Region")),
    ("522", As_U.Tus ("CN-SC"),
            As_U.Tus ("Sichuan Province")),
    ("523", As_U.Tus ("CN-YN"),
            As_U.Tus ("Yunnan Province")),
    ("524", As_U.Tus ("CN-XZ"),
            As_U.Tus ("Xizang Autonomous Region (Tibet)")),
    ("525", As_U.Tus ("CN-GS"),
            As_U.Tus ("Gansu Province")),
    ("526", As_U.Tus ("CN-QH"),
            As_U.Tus ("Qinghai Province (Tsinghai Province)")),
    ("527", As_U.Tus ("CN-XJ"),
            As_U.Tus ("Xinjiang Uyghur Autonomous Region")),
    ("528", As_U.Tus ("CHN"),
            As_U.Tus ("China (People's Republic of _)")),
    ("529", As_U.Tus ("UMI"),
            As_U.Tus ("United States Minor Outlying Islands")),
    ("530", As_U.Tus ("CPT"),
            As_U.Tus ("Clipperton Island")),
    ("531", As_U.Tus ("ATA"),
            As_U.Tus ("Antarctica")),
    ("532", As_U.Tus ("AAA"),
            As_U.Tus ("International (Worldwide) (Earth)"))
  );

end Countries;

